# Copyright (C) 2019-2020  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import pkg_resources
import os


def get_sentry_release():
    main_package = os.environ.get('SWH_MAIN_PACKAGE')
    if main_package:
        version = pkg_resources.get_distribution(main_package).version
        return f'{main_package}@{version}'
    else:
        return None


def init_sentry(
        sentry_dsn, *, debug=None, integrations=[],
        extra_kwargs={}):
    if debug is None:
        debug = bool(os.environ.get('SWH_SENTRY_DEBUG'))
    sentry_dsn = sentry_dsn or os.environ.get('SWH_SENTRY_DSN')

    if sentry_dsn:
        import sentry_sdk

        sentry_sdk.init(
            release=get_sentry_release(),
            dsn=sentry_dsn,
            integrations=integrations,
            debug=debug,
            **extra_kwargs,
        )



# Copyright (C) 2018-2019  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import re
import pytest

from swh.core.api import remote_api_endpoint, RPCClient


@pytest.fixture
def rpc_client(requests_mock):
    class TestStorage:
        @remote_api_endpoint('test_endpoint_url')
        def test_endpoint(self, test_data, db=None, cur=None):
            return 'egg'

        @remote_api_endpoint('path/to/endpoint')
        def something(self, data, db=None, cur=None):
            return 'spam'

    class Testclient(RPCClient):
        backend_class = TestStorage

    def callback(request, context):
        assert request.headers['Content-Type'] == 'application/x-msgpack'
        context.headers['Content-Type'] = 'application/x-msgpack'
        if request.path == '/test_endpoint_url':
            context.content = b'\xa3egg'
        elif request.path == '/path/to/endpoint':
            context.content = b'\xa4spam'
        else:
            assert False
        return context.content

    requests_mock.post(re.compile('mock://example.com/'),
                       content=callback)

    return Testclient(url='mock://example.com')


def test_client(rpc_client):

    assert hasattr(rpc_client, 'test_endpoint')
    assert hasattr(rpc_client, 'something')

    res = rpc_client.test_endpoint('spam')
    assert res == 'egg'
    res = rpc_client.test_endpoint(test_data='spam')
    assert res == 'egg'

    res = rpc_client.something('whatever')
    assert res == 'spam'
    res = rpc_client.something(data='whatever')
    assert res == 'spam'



# Copyright (C) 2019  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import os
import pkg_resources
from unittest.mock import patch

import swh.core.api.gunicorn_config as gunicorn_config


def test_post_fork_default():
    with patch('sentry_sdk.init') as sentry_sdk_init:
        gunicorn_config.post_fork(None, None)

    sentry_sdk_init.assert_not_called()


def test_post_fork_with_dsn_env():
    flask_integration = object()  # unique object to check for equality
    with patch('sentry_sdk.integrations.flask.FlaskIntegration',
               new=lambda: flask_integration):
        with patch('sentry_sdk.init') as sentry_sdk_init:
            with patch.dict(os.environ, {'SWH_SENTRY_DSN': 'test_dsn'}):
                gunicorn_config.post_fork(None, None)

    sentry_sdk_init.assert_called_once_with(
        dsn='test_dsn',
        integrations=[flask_integration],
        debug=False,
        release=None,
    )


def test_post_fork_with_package_env():
    flask_integration = object()  # unique object to check for equality
    with patch('sentry_sdk.integrations.flask.FlaskIntegration',
               new=lambda: flask_integration):
        with patch('sentry_sdk.init') as sentry_sdk_init:
            with patch.dict(os.environ, {'SWH_SENTRY_DSN': 'test_dsn',
                                         'SWH_MAIN_PACKAGE': 'swh.core'}):
                gunicorn_config.post_fork(None, None)

    version = pkg_resources.get_distribution('swh.core').version

    sentry_sdk_init.assert_called_once_with(
        dsn='test_dsn',
        integrations=[flask_integration],
        debug=False,
        release='swh.core@' + version,
    )


def test_post_fork_debug():
    flask_integration = object()  # unique object to check for equality
    with patch('sentry_sdk.integrations.flask.FlaskIntegration',
               new=lambda: flask_integration):
        with patch('sentry_sdk.init') as sentry_sdk_init:
            with patch.dict(os.environ, {'SWH_SENTRY_DSN': 'test_dsn',
                                         'SWH_SENTRY_DEBUG': '1'}):
                gunicorn_config.post_fork(None, None)

    sentry_sdk_init.assert_called_once_with(
        dsn='test_dsn',
        integrations=[flask_integration],
        debug=True,
        release=None,
    )


def test_post_fork_no_flask():
    with patch('sentry_sdk.init') as sentry_sdk_init:
        with patch.dict(os.environ, {'SWH_SENTRY_DSN': 'test_dsn'}):
            gunicorn_config.post_fork(None, None, flask=False)

    sentry_sdk_init.assert_called_once_with(
        dsn='test_dsn',
        integrations=[],
        debug=False,
        release=None,
    )


def test_post_fork_extras():
    flask_integration = object()  # unique object to check for equality
    with patch('sentry_sdk.integrations.flask.FlaskIntegration',
               new=lambda: flask_integration):
        with patch('sentry_sdk.init') as sentry_sdk_init:
            with patch.dict(os.environ, {'SWH_SENTRY_DSN': 'test_dsn'}):
                gunicorn_config.post_fork(
                    None, None, sentry_integrations=['foo'],
                    extra_sentry_kwargs={'bar': 'baz'})

    sentry_sdk_init.assert_called_once_with(
        dsn='test_dsn',
        integrations=['foo', flask_integration],
        debug=False,
        bar='baz',
        release=None,
    )
